import de.humatic.dsj.*;
import de.humatic.dsj.com.*;

public class AviExporter implements java.beans.PropertyChangeListener {

	/* Shows how to get instances of the various IAM... COM interfaces to configure details of video encoding **/

	private boolean done;

	private DSMovie m ;

	private DSFilter vEnc,
					 aEnc;

	private DSFilter.DSPin vOut,
						   aOut,
						   vEncOut,
						   aEncOut;

	private String[] videoEncoders = new String[]{"MJPEG Compressor",                   // stock Windows codec, 32 & 64bit
												  "Cinepak Codec By Radius",			// stock Windows codec, 32 bit only
												  "Xvid MPEG-4 Codec", 					// http://www.xvid.org/Downloads.15.0.html
												  "WebM VP8 Encoder Filter", 			// http://www.optimasc.com/products/vp8vfw/index.html
												  "DV Video Encoder"					// stock Windows codec, 32 & 64bit
												  };

	public AviExporter() {}

	void createGraph() {

		m = new DSMovie("media\\2997ndHalfNTSC.wmv", DSFiltergraph.HEADLESS | DSFiltergraph.INIT_PAUSED | DSFiltergraph.NO_SYNC, this);

		m.stop();

		DSFilter videoRenderer = m.findRenderer(DSMediaType.MT_VIDEO);

		vOut = videoRenderer.getPin(1, 0).connectedTo();

		m.removeFilter(videoRenderer, false);

		insertAndConfigureVideoCodec(videoEncoders[0]);

		DSFilter audioRenderer = m.findRenderer(DSMediaType.MT_AUDIO);

		aOut = audioRenderer.getPin(1, 0).connectedTo();

		m.removeFilter(audioRenderer, false);

		DSFilterInfo aEncInfo = DSFilterInfo.filterInfoForName("Microsoft ADPCM");

		DSFilter aEnc = m.addFilterToGraph(aEncInfo);

		DSFilter.DSPin aEncIn = aEnc.getPin(1, 0);

		try{ aOut.getFilter().connectDownstream(aOut, aEncIn, true); } catch (DSJException de) {

			if (de.getErrorCode() == 0x80040217 || de.getErrorCode() == 0x80040224) {

				m.stop();

				aOut.getFilter().connectDownstream(aOut, aEncIn, true);

			} else throw(de);

		}

		DSFilter aviMux = m.addFilterToGraph(DSFilterInfo.filterInfoForName("AVI Mux"));

		final IConfigAviMux icam = (IConfigAviMux)(COMFactory.queryInterface(m, aviMux, COMFactory.IID_IConfigAviMux));

		icam.setMasterStream(0);

		icam.setCompatibilityIndex(true);

		final IConfigInterleaving ici = (IConfigInterleaving)(COMFactory.queryInterface(m, aviMux, COMFactory.IID_IConfigInterleaving));

		ici.setMode(IConfigInterleaving.INTERLEAVE_NONE);

		ici.setInterleaving(500, 50);

		DSFilter.DSPin aEncOut = aEnc.getPin(0, 0);

		aEnc.connectDownstream(aEncOut, aviMux.getPin(1, 0), false);

		DSFilter.DSPin muxIn = aviMux.getPin(1, 1);

		vEnc.connectWithFormat(vEncOut, -1, muxIn);

		DSFilter fileWriter = m.addFilterToGraph(DSFilterInfo.filterInfoForName("File Writer"));

		fileWriter.setParameter("src", "media\\output.avi");

		aviMux.connectDownstream(aviMux.getPin(0, 0), fileWriter.getPin(1, 0), true);

		m.dumpGraph(false);

		m.play();

		while (!done) {

			System.out.print(m.getTime()+"/"+m.getDuration()+"\r");

			try{ Thread.currentThread().sleep(1000); } catch (Exception e){}

		}

	}

	private void insertAndConfigureVideoCodec(String cName) {

		DSFilterInfo vEncInfo = DSFilterInfo.filterInfoForName(cName);

		vEnc = m.addFilterToGraph(vEncInfo);

		DSFilter.DSPin vEncIn = vEnc.getPin(1, 0);

		try{ vOut.getFilter().connectDownstream(vOut, vEncIn, false); } catch (DSJException de) {

			if (de.getErrorCode() == 0x80040216) {

				/* encoder uses ICM Image Compression Manager with a proxy filter */

				DSFilter converter = m.findFilterByName("AVI Decompressor");

				if (converter != null) {

					m.removeFilter(converter, false);

					m.stop();

					vOut.getFilter().connectDownstream(vOut, vEncIn, false);

				}

			} else if (de.getErrorCode() == 0x80040217 || de.getErrorCode() == 0x80040224) {

				m.stop();

				vOut.getFilter().connectDownstream(vOut, vEncIn, false);

			} else throw(de);

		}

		vEncOut = vEnc.getPin(0, 0);

		DSMediaType[] streamTypes = vEncOut.getAvailableOutputTypes();

		for (int i = 0; i < streamTypes.length; i++) System.out.println(streamTypes[i]);

		try{

			final IAMVideoCompression ivc = (IAMVideoCompression)(COMFactory.queryInterface(m, vEncOut, COMFactory.IID_IAMVideoCompression));

			System.out.println("IAMVideoCompression\n   Capabilities: "+ivc.getCapabilites()+"\n   Quality: "+ivc.getQuality()+", def.: "+ivc.getDefaultQuality()+"\n   Keyframe rate: "+ivc.getKeyframeRate()+", def: "+ivc.getDefaultKeyframeRate());

			ivc.setQuality(0.2f);

			ivc.setKeyframeRate(10);

			System.out.println("   set Quality: "+ivc.getQuality()+"\n   set Keyframe rate: "+ivc.getKeyframeRate());

		} catch (Exception ex){ ex.printStackTrace(); }

		try{

			final IAMStreamConfig isc = (IAMStreamConfig)(COMFactory.queryInterface(m, vEncOut, COMFactory.IID_IAMStreamConfig));

			System.out.println("IAMStreamConfig");

			DSMediaType dsmt = isc.getFormat(0);

			int numCaps = isc.getNumberOfCapabilities();

			System.out.println("   num caps: "+numCaps+"\n   current format: "+dsmt);

			isc.setFormat(dsmt);

		} catch (Exception ex){ex.printStackTrace(); }

		try{

			final IDVEnc idv = (IDVEnc)(COMFactory.queryInterface(m, vEnc, COMFactory.IID_IDVEnc));

			idv.setDVFormat(2000, 2009, 2015, null);

		} catch (Exception ex){ex.printStackTrace(); }

		vEnc.showPropertiesDialog();

	}


	public void propertyChange(java.beans.PropertyChangeEvent pe) {

		switch (DSJUtils.getEventType(pe)) {

			case DSFiltergraph.DONE:

				done = true;

				System.out.println("DONE");

				m.stop();

				m.dispose();

				break;

		}
	}

	public static void main(String[] args) {

		new AviExporter().createGraph();

	}

}